import sys
import typing

GenericType = typing.TypeVar("GenericType")


def cleanup_po(args, settings):
    ''' 

    '''

    pass


def language_menu(args, settings):
    ''' 

    '''

    pass


def main():
    ''' 

    '''

    pass


def rtl_process_po(args, settings):
    ''' 

    '''

    pass


def strip_po(args, settings):
    ''' 

    '''

    pass


def update_po(args, settings):
    ''' 

    '''

    pass
